"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Markup_1 = require("./Markup");
var MEDIUM_IMG_CDN = 'https://cdn-images-1.medium.com';
var MEDIUM_MEDIA = 'https://medium.com/media';
var Paragraph = /** @class */ (function () {
    /**
     * @param  {string} publicname
     * @param  {number} publictype
     * @param  {string} publictext
     * @param  {Markup[]} publicmarkups
     */
    function Paragraph(json) {
        this.type = setParagraphType(json.type);
        this.name = json.name;
        this.markups = json.markups ? json.markups.map(function (m) { return new Markup_1.default(m); }) : []; // eslint-disable-line
        this.text = json.text;
        this.metadata = json.metadata;
        this.iframe = json.iframe;
    }
    Object.defineProperty(Paragraph.prototype, "hasAnyMarkups", {
        get: function () {
            return this.markups.length > 0;
        },
        enumerable: true,
        configurable: true
    });
    Paragraph.prototype.processParagraph = function () {
        var markups = createMarkupArray(this.markups);
        var parsedText = this.text;
        if (markups.length > 0) {
            var previousIndex = 0;
            var text = this.text;
            var tokens = [];
            for (var j = 0; j < markups.length; j++) {
                if (markups[j]) {
                    var token = text.substring(previousIndex, j);
                    previousIndex = j;
                    tokens.push(token);
                    tokens.push(markups[j]);
                }
            }
            tokens.push(text.substring(markups.length - 1));
            parsedText = tokens.join('');
        }
        return this.addParagraphMarkup(parsedText);
    };
    Paragraph.prototype.addParagraphMarkup = function (parsedText) {
        var text;
        switch (this.type) {
            case ParagraphType.Standard:
                text = "\n " + parsedText;
                break;
            case ParagraphType.H1:
                text = "\n# " + parsedText.replace(/\n/g, '\n# ');
                break;
            case ParagraphType.ImageAndCaption:
                var imgsrc = MEDIUM_IMG_CDN;
                if (this.metadata.originalHeight) {
                    imgsrc = imgsrc + "/fit/" + this.metadata.originalWidth + "/" + this.metadata.originalHeight + "/" + this.metadata.id;
                }
                else {
                    imgsrc = imgsrc + "/max/" + this.metadata.originalWidth + "/" + this.metadata.id;
                }
                text = "\n![" + parsedText + "](" + imgsrc + ")";
                if (parsedText) {
                    text += "*" + parsedText + "*";
                }
                break;
            case ParagraphType.BlockQuote:
                text = "\n> " + parsedText;
                break;
            case ParagraphType.Code:
                text = "```\n " + parsedText + "```";
                break;
            case ParagraphType.UnOrderedList:
                text = "\n* " + parsedText;
                break;
            case ParagraphType.OrderedList:
                text = "\n1. " + parsedText;
                break;
            case ParagraphType.IFrame:
                // TODO: use youtube videos if they have it
                var iframeSrc = MEDIUM_MEDIA + "/" + this.iframe.mediaResourceId;
                // text = `[![${parsedText}](${MEDIUM_MEDIA}/${this.iframe.mediaResourceId}/0.jpg)](http://www.youtube.com/watch?v=YOUTUBE_VIDEO_ID_HERE)`
                text = "<iframe src=\"" + iframeSrc + "\" frameborder=0></iframe>";
                break;
            case ParagraphType.SubTitle:
                text = "\n### " + parsedText;
                break;
        }
        return "\n" + text;
    };
    return Paragraph;
}());
exports.default = Paragraph;
function createMarkupArray(markups) {
    var array = [];
    markups.map(function (m) {
        if (array[m.start]) {
            array[m.start] += m.open;
        }
        else {
            array[m.start] = m.open;
        }
        if (array[m.end]) {
            if (m.type === Markup_1.MarkupType.Anchor) {
                array[m.end] += array[m.end];
            }
            else {
                array[m.end] = m.close + array[m.end];
            }
        }
        else {
            array[m.end] = m.close;
        }
    });
    return array;
}
function setParagraphType(type) {
    switch (type) {
        case 1:
            return ParagraphType.Standard;
        case 2:
        case 3:
            return ParagraphType.H1;
        case 4:
            return ParagraphType.ImageAndCaption;
        case 6:
            return ParagraphType.BlockQuote;
        case 8:
            return ParagraphType.Code;
        case 9:
            return ParagraphType.UnOrderedList;
        case 10:
            return ParagraphType.OrderedList;
        case 11:
            return ParagraphType.IFrame;
        case 13:
            return ParagraphType.SubTitle;
        default:
            throw new Error("Unknown paragraph type \"" + type + "\" encountered.");
    }
}
var ParagraphType;
(function (ParagraphType) {
    ParagraphType[ParagraphType["Standard"] = 1] = "Standard";
    ParagraphType[ParagraphType["H1"] = 2] = "H1";
    ParagraphType[ParagraphType["ImageAndCaption"] = 3] = "ImageAndCaption";
    ParagraphType[ParagraphType["BlockQuote"] = 4] = "BlockQuote";
    ParagraphType[ParagraphType["Code"] = 5] = "Code";
    ParagraphType[ParagraphType["UnOrderedList"] = 6] = "UnOrderedList";
    ParagraphType[ParagraphType["IFrame"] = 7] = "IFrame";
    ParagraphType[ParagraphType["SubTitle"] = 8] = "SubTitle";
    ParagraphType[ParagraphType["OrderedList"] = 9] = "OrderedList";
})(ParagraphType = exports.ParagraphType || (exports.ParagraphType = {}));
//# sourceMappingURL=Paragraph.js.map